use hmac::{Hmac, Mac};
use sha2::Sha256;

pub struct CloudSecurity {
    login_key: &'static str,
    iot_key: &'static str,
    hmac_key: &'static str,

    access_token: Option<String>,
    random_data: Option<String>,
}

impl CloudSecurity {
    pub fn new(login_key: &'static str, iot_key: &'static str, hmac_key: &'static str) -> Self {
        Self {
            login_key,
            iot_key,
            hmac_key,

            access_token: None,
            random_data: None,
        }
    }

    pub fn set_aes_keys(&mut self, access_token: &str, random_data: &str) {
        self.access_token = Some(access_token.to_string());
        self.random_data = Some(random_data.to_string());
    }

    pub fn sign(&self, data: &str, random: &str) -> String {
        let mut msg = self.iot_key.to_string();

        msg += data;
        msg += random;

        type HmacSha256 = Hmac<Sha256>;

        let mut mac = HmacSha256::new_from_slice(self.hmac_key.as_bytes()).unwrap();
        mac.update(msg.as_bytes());

        mac.finalize()
            .into_bytes()
            .iter()
            .map(|b| format!("{b:02x}"))
            .collect::<Vec<String>>()
            .join("")
    }
}

#[cfg(test)]
mod test {
    use crate::cloud::Cloud;

    use super::*;
}
